﻿#include "distance.h"

float
euclideanSqAVX(
        const u_int8_t* RESTRICT a,
        const u_int8_t* RESTRICT b,
        u_int32_t length)
{
    assert(length % 32 == 0);

    __m256i zer256 = _mm256_setzero_si256();
    __m256i accReg = _mm256_setzero_si256();

    ALIGNED(32) u_int32_t res32[8];

    u_int32_t i;
    for(i = 0u; i < length; i += 32u)
    {
        __m256i  vd, vs;

        __m256i va = _mm256_loadu_si256((__m256i*)&a[i]);
        __m256i vb = _mm256_loadu_si256((__m256i*)&b[i]);

        vd = _mm256_abs_epi8(_mm256_sub_epi8(va, vb));
        vs = _mm256_maddubs_epi16(vd, vd);
        accReg = _mm256_add_epi16(accReg, vs);

    }

    __m256i hi = _mm256_unpackhi_epi16(accReg, zer256);
    __m256i lo = _mm256_unpacklo_epi16(accReg, zer256);
    accReg = _mm256_hadd_epi32(hi, lo);
    accReg = _mm256_hadd_epi32(accReg, zer256);
    _mm256_store_si256((__m256i*)&res32[0], accReg);
    float acc = res32[0] + res32[1] + res32[4] + res32[5];

    acc /= (128*128);

    return acc;
}
