/*
 * User Defined Function VLMatch: measures squared distance between two byte vectors of the fixed length
 *
 * Create Date: Oct 24, 2019
 */

#include "postgres.h"
#include "fmgr.h"

#include "distance.h"


#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

PG_FUNCTION_INFO_V1(VLMatch);

// This is a function that measures squared distance between two byte vectors of the fixed length
Datum
VLMatch(PG_FUNCTION_ARGS) {
    bytea *arg1 = PG_GETARG_BYTEA_P(0);
    bytea *arg2 = PG_GETARG_BYTEA_P(1);
    u_int32_t descriptorLength = PG_GETARG_INT32(2);

    // Get input binaries.
    uint8 *descriptorFst = (uint8 *) arg1->vl_dat;
    uint8 *descriptorSnd = (uint8 *) arg2->vl_dat;

    // Match descriptors.
    float8 res = euclideanSqAVX(descriptorFst, descriptorSnd, descriptorLength);

    PG_RETURN_FLOAT8(res);
}
